import pysam
from tqdm import tqdm
from typing import Set, Mapping, List


def extract_interested_channels(bam_filepath: str, min_length: int):
    channels = set()
    with pysam.AlignmentFile(
        filename=bam_filepath, mode="rb", check_sq=False, threads=40
    ) as bam_file:
        for read in tqdm(
            bam_file.fetch(until_eof=True), desc=f"reading {bam_filepath}"
        ):
            if read.query_name.endswith("sbr"):
                continue
            
            if read.query_length >= min_length:
                channels.add(read.get_tag("ch"))
    return channels


def expand_channels(channels: Set[int], expected_num: int) -> Mapping[int, List[int]]:
    result = {}
    for ch in channels:
        result.setdefault(ch, [ch])

    cur_num = len(channels)

    sorted_channels = sorted(list(channels))
    cursor = 0

    for new_ch in range(100000000):
        if cur_num >= expected_num:
            break
        if new_ch in channels:
            continue

        ch_idx = cursor % len(channels)
        cursor += 1
        cur_num += 1

        anchor_ch = sorted_channels[ch_idx]
        result[anchor_ch].append(new_ch)
    return result


def modify_read_info(read_old: pysam.AlignedSegment, old_ch: int, new_ch: int):
    idx = read_old.query_name.rsplit("/", maxsplit=1)[1]
    read_old.query_name = f"read_{new_ch}/{new_ch}/subread/{idx}"
    read_old.set_tag("ch", new_ch, value_type="i")
    return read_old


def dumping_new_subreads(sbr_bam_filepath: str, channels: Mapping[int, List[int]]):
    o_bam_filename = "{}.{}.bam".format(
        sbr_bam_filepath.rsplit(".", maxsplit=1)[0], "expanded-2"
    )
    with pysam.AlignmentFile(
        sbr_bam_filepath, mode="rb", threads=40, check_sq=False
    ) as in_bam:
        with pysam.AlignmentFile(
            o_bam_filename, mode="wb", threads=40, check_sq=False, header=in_bam.header
        ) as out_bam:

            for record in tqdm(
                in_bam.fetch(until_eof=True), desc=f"dumping {o_bam_filename}"
            ):
                ch = int(record.get_tag("ch"))
                if ch in channels:
                    old_ch = ch
                    for new_ch in channels[ch]:
                        record = modify_read_info(
                            read_old=record, old_ch=old_ch, new_ch=new_ch
                        )
                        out_bam.write(record)
                        old_ch = new_ch


def main_cli():

    sbr_bam_filepath = "/data/ccs_data/speed-test/8k/haorui_hla_subreads.bam"
    smc_bam_filepath = "/data/ccs_data/speed-test/8k//haorui_hla_subreads.smc_all_reads.bam"

    channels = extract_interested_channels(smc_bam_filepath, min_length=8000)
    channels = expand_channels(channels=channels, expected_num=500000)
    dumping_new_subreads(sbr_bam_filepath=sbr_bam_filepath, channels=channels)
    


if __name__ == "__main__":
    main_cli()



from collections import Counter
from typing import List
import json
from tqdm import tqdm
import seaborn as sns
import matplotlib.pyplot as plt

def load_data(file_path: str):
    with open(file_path, "r") as file:
        for line in tqdm(file):
            line = line.strip()
            v = json.loads(line)
            qual = v["msa_seqs"][0]
            assert isinstance(qual, str)
            qual = qual.replace("-", "9")
            qual = qual.replace("#", "9")
            sbrs = v["msa_seqs"][3:]
            yield qual, sbrs

def count_alternating_columns(q_values: str, surbeads: List[str]) -> int:
    # 将 Q 值字符串转换为整数列表
    q_values_list = list(map(int, q_values.strip()))

    # 存储满足条件的列数
    alternating_count = 0

    # 遍历每一列
    for col_index in range(len(q_values_list)):
        if q_values_list[col_index] < 4:  # 判断 Q 值是否小于 4
            # 提取当前列的碱基
            column_bases = [surbead[col_index] for surbead in surbeads if col_index < len(surbead)]

            # 计算碱基的频次
            base_counts = Counter(column_bases)
            total_count = sum(base_counts.values())

            # 如果出现的碱基数小于 2，不可能交替
            if len(base_counts) < 2:
                continue

            # 获取出现频次最高的两个碱基及其比例
            most_common = base_counts.most_common(2)
            top1_base, top1_count = most_common[0]
            top2_base, top2_count = most_common[1]

            top1_ratio = top1_count / total_count
            top2_ratio = top2_count / total_count

            # 检查两者的比例是否都在 40% 以上
            if top1_ratio >= 0.45 and top2_ratio >= 0.45:
                alternating_count += 1

    return alternating_count

def main():
    file_path = "/data/ccs_data/case-study/20250310-lowQ30/sbr2icing.q30.asrtc.txt"
    locus = 0
    channels = 0
    
    in_consis_cnts = []
    
    for q_values, surbeads in load_data(file_path):
        result = count_alternating_columns(q_values, surbeads)
        if result > 0 and result < 50:
            in_consis_cnts.append(min(result, 100))
        locus += result
        if result > 0:
            channels += 1
    print(locus, channels)
    plt.figure(figsize=(10, 6))
    sns.histplot(in_consis_cnts, bins=50, kde=False)
    plt.title('Inconsistent Counts Distribution')
    plt.xlabel('Inconsistent Count')
    plt.ylabel('Frequency')
    plt.xticks(range(0, 50, 1))
    plt.xticks(rotation=45)

    plt.savefig('in_consis_cnts.png')
    plt.close()
            

if __name__ == "__main__":
    main()



"""
将多个 adapter.bam 和 smc_all_reads.bam 文件合并成一个. dw 和 ar 特征需要从 called bam 中进行透传
"""

def main():
    adapter_bams = [
        "/data/ccs_data/little-mouse/20250515_240601Y0005_Run0001_called.bam",
        "/data/ccs_data/little-mouse/20250515_240601Y0005_Run0002_called.bam"
    ]
    
    sbr_bams = [
        "/data/ccs_data/little-mouse/20250515_240601Y0005_Run0001_adapter.bam",
        "/data/ccs_data/little-mouse/20250515_240601Y0005_Run0002_adapter.bam"
    ]
    
    smc_bams = [
        "/data/ccs_data/little-mouse/20250515_240601Y0005_Run0001-rerun.smc_all_reads.bam",
        "/data/ccs_data/little-mouse/20250515_240601Y0005_Run0002-rerun.smc_all_reads.bam"
    ]
    
    
    pass

if __name__ == "__main__":
    pass
